
function his = umereader(file);

%
% function his = umereader(file);
%
% HIS .ume file reader (i.e. 100 word header files)
%
% calling sequence is i.e. "his = umereader;"
%
% optional input is "file", the input .ume file name.
% 
% output is the structure "his" containing wavenumbers, the header,
% radiances, and specific header fields are also defined.
%
% warnings/bugs: 1) for use w/ browser.m, 1900 is added to the year.
%                2) does almost no error checking
%
% author: Dave Tobin on 10/31/97.  email: dave.tobin@ssec.wisc.edu
%

warning off

% if not input, get filename 
if nargin == 0 
  [fname,pname] = uigetfile;
  file = [pname fname];
end
his.fileName = file;

% Here are the guts of the program.  
% open the file, read the data, and close the file
fid = fopen(file,'r','l');
data = fread(fid,[2150, inf],'float32');
fclose(fid);

% get number of spectra and take transpose of data
nspec=size(data);nspec=nspec(2);
data = data';

% grab the 100 word header
header = data(1:nspec,1:100);

npts = header(1,31);

% define some specific header data
if 1
  his.record_number = header(:,1);
  his.channel_number = header(:,2);
  his.mirror_position = header(:,3);
  his.sceneMirrorAngle = header(:,3);
  his.no_pts_in_ifgm = header(:,8);
  his.seconds = header(:,11);
  his.hbb_temp = header(:,12);
  his.cbb_temp = header(:,13);
  his.dc_ch1 = header(:,14);
  his.dc_ch2 = header(:,15);
  his.dc_ch3 = header(:,16);
  his.year = header(:,17) + 1900;
  his.month = header(:,18);
  his.day = header(:,19);
  his.bad_flag = header(:,20);
  his.noise_flag = header(:,21);
  his.std_phase = header(:,22);
  his.begin_time = header(:,23);   % HHMMSS
  his.current_time = header(:,24); % HHMMSS
  his.qc_flag = header(:,25);      % 2 = spectrum o.k.
  his.no_pts_in_fft = header(:,30);
  his.no_pts_in_spectrum = header(:,31);
  his.aircraft_lat = header(:,51);
  his.aircraft_lon = header(:,52);
  his.aircraft_alt = header(:,53);
  his.aircraft_heading = header(:,54);
end

% grab the radiances
his.Radiance = data(1:nspec,101:101+npts-1);

% Create frequency scale.
low = header(1,34);
high = header(1,35);
his.wnum1 = linspace(low,high,npts)';

% pull hour, minute, second from current_time (HHMMSS)
his.hour = fix(his.current_time/1e4);
his.minute = fix((his.current_time - 1e4*his.hour)/1e2);
his.second =  fix(his.current_time - his.hour*1e4 - his.minute*1e2);
his.timeHHMMSS = his.current_time;

% compute # seconds since 1970
his.nss1970=zeros(length(his.year),1);
for i = 1:length(his.year)
  his.nss1970(i) = nosecsince1970(his.year(i),his.month(i),...
     his.day(i),his.hour(i),his.minute(i),his.second(i));
end

his.index = (1:length(his.record_number))';
his.Latitude=his.aircraft_lat;
his.Longitude=his.aircraft_lon;
his.Altitude = his.aircraft_alt;
his.time_offset=his.nss1970;
his.base_time=0;
his.Time=his.timeHHMMSS;
his.dateNumber=dn(his.nss1970);





